import { TransferCtrl } from "@/modules/editor/controllers/TransferCtrl";
import { css } from "@linaria/core";
import { defineComponent, onMounted, ref } from "vue";
import { useEditor } from "../../../..";
import { string } from "vue-types";
import { CompToolbars } from "@/modules/editor/objects/EditingCompTools";
import { DesignComp } from "@/modules/editor/objects/DesignTemp/DesignComp";

const btnStyles = {
  top: {
    top: 0,
    left: "50%",
    transform: "translate(-50%, -50%)",
  },
  bottom: {
    bottom: 0,
    left: "50%",
    transform: "rotate(180deg) translate(50%, -50%)",
  },
  left: {
    top: "50%",
    left: 0,
    transform: "rotate(-90deg) translate(50%, -50%)",
  },
  right: {
    top: "50%",
    right: 0,
    transform: "rotate(90deg) translate(-50%, -50%)",
  },
};

export const Transfer = defineComponent({
  props: {
    compId: string().isRequired,
  },
  setup(props) {
    const transferRef = ref();
    const editor = useEditor();
    const transferCtrl = new TransferCtrl(editor);

    onMounted(() => {
      transferCtrl.init(
        transferRef.value.parentElement as HTMLElement,
        props.compId
      );
    });

    return () => {
      const comp = editor.helper.findComp(props.compId) as DesignComp;
      return (
        <div ref={transferRef}>
          <div class={toolbarStyle}>
            {CompToolbars.default.map((item, i) => {
              return (
                <item.component
                  class="p-4px"
                  value={item.value?.(comp)}
                  onClick={() => item.onClick.call(editor, comp)}
                />
              );
            })}
          </div>
          <div class={borderStyle}></div>
          <div
            class={[resizeStyle, "drag-disable"]}
            onMousedown={(e) => transferCtrl.mousedown(e, "resize")}
          ></div>
          {Object.entries(btnStyles).map(([name, style]) => (
            <div
              class={[fanBtnStyle, "drag-disable"]}
              style={style}
              onMousedown={(e) =>
                transferCtrl.mousedown(
                  e,
                  `offset_${name as keyof typeof btnStyles}`
                )
              }
            >
              ↑
            </div>
          ))}
        </div>
      );
    };
  },
});

const borderStyle = css`
  position: absolute;
  top: 0;
  left: 0;
  width: 100%;
  height: 100%;
  outline: 2px solid @inf-primary-color;
  pointer-events: none;
  z-index: 9;
`;

const resizeStyle = css`
  position: absolute;
  bottom: 0;
  right: 0;
  width: 16px;
  height: 16px;
  border-radius: 50%;
  background-color: #fff;
  z-index: 9;
  transform: translate(50%, 50%);
  box-shadow: 0 0 2px 2px rgba(0, 0, 0, 0.2);
  cursor: nwse-resize;
  &:hover {
    border-color: @inf-primary-color;
  }
`;

const fanBtnStyle = css`
  position: absolute;
  width: 30px;
  height: 30px;
  border-radius: 50%;
  background-color: rgba(255, 255, 255, 0.3);
  font-size: 12px;
  text-align: center;
  clip-path: polygon(0 0, 100% 0, 100% 50%, 0 50%);
  cursor: grab;
  z-index: 99;

  &:hover {
    background: @inf-primary-color;
  }
`;

const toolbarStyle = css`
  @apply bg-white shadow rounded space-x-4px p-4px whitespace-nowrap;
  position: absolute;
  top: 0;
  left: 50%;
  transform: translate(-50%, -60px);
`;
