import { DesignComp } from "@/modules/editor/objects/DesignTemp/DesignComp";
import { CompUI } from "..";
import { mapValuesDeep } from "@/utils";
import { Exception } from "queenjs";

const cacheCompMap = new Map<string, DesignComp>();

export function addCacheToMap(
  obj: Record<string, DesignComp>,
  parentId: string
) {
  let lastComp: DesignComp | undefined;
  cacheCompMap.forEach((comp) => {
    obj[comp.id] = comp;
    lastComp = comp;
  });
  if (lastComp) {
    setCompPid(lastComp, parentId);
  }
  cacheCompMap.clear();
}

export function createCompId(
  compKey: keyof typeof CompUI,
  options?: any
): string {
  const comp = CompUI[compKey].createComp(
    Object.assign(options || {}, { compKey })
  );
  cacheCompMap.set(comp.id, comp);

  const childIds = mapValuesDeep(
    comp.children,
    (v) => typeof v === "string",
    (v: string) => v
  );

  childIds.forEach((id) => {
    const childComp = cacheCompMap.get(id);
    if (!childComp) {
      throw Exception.error("无效的childCompId");
    }
    setCompPid(childComp, comp.id);
  });
  return comp.id;
}

export function setCompPid(comp: DesignComp, pid: string) {
  Object.defineProperty(comp, "pid", {
    value: pid,
  });
}
