import { ModuleControl } from "queenjs";
import { EditorModule } from "../../module";
import * as transforms from "./transforms";
import { DesignComp } from "../../objects/DesignTemp/DesignComp";

type TransHandle = (this: TransferCtrl, e: MouseEvent) => void;

export type TransCreateFn = (...args: any[]) => {
  mousedown?: TransHandle;
  mousemove: TransHandle;
  mouseup: TransHandle;
};

export type TransEvent = {
  startX: number;
  startY: number;
  offsetX: number;
  offsetY: number;
  width: number;
  height: number;
};

export class TransferCtrl extends ModuleControl<EditorModule> {
  compEl!: HTMLElement;
  currComp!: DesignComp;
  transEvent!: TransEvent;
  transforms = transforms;
  currTransfer!: ReturnType<TransCreateFn>;
  init(el: HTMLElement, compId: string) {
    this.compEl = el;
    this.currComp = this.helper.findComp(compId) as DesignComp;
  }
  mousedown(e: MouseEvent, type: keyof typeof transforms) {
    const { currComp } = this.store;
    this.transEvent = {
      startX: e.clientX,
      startY: e.clientY,
      offsetX: 0,
      offsetY: 0,
      width: currComp.layout.size?.[0] || this.compEl.clientWidth * 2,
      height: currComp.layout.size?.[1] || this.compEl.clientHeight * 2,
    };
    this.currTransfer = this.transforms[type]();
    document.addEventListener("mousemove", this.mousemove);
    document.addEventListener("mouseup", this.mouseup);
    this.currTransfer.mousedown?.call(this, e);
  }
  private mousemove = (e: MouseEvent) => {
    const { transEvent } = this;
    transEvent.offsetX = e.clientX - transEvent.startX;
    transEvent.offsetY = e.clientY - transEvent.startY;
    this.currTransfer.mousemove.call(this, e);
  };
  private mouseup = (e: MouseEvent) => {
    document.removeEventListener("mousemove", this.mousemove);
    document.removeEventListener("mouseup", this.mouseup);
    this.currTransfer.mouseup.call(this, e);
  };
}
