import { TransCreateFn, TransferCtrl } from "..";

type Direction = "top" | "bottom" | "x";

const offset: TransCreateFn = (direction: Direction) => ({
  mousemove() {
    if (this.currComp.layout.margin) return;

    const style = this.module.helper.createStyle({
      offset: {
        [direction[0] as "x"]: getOffset.call(this, direction),
      },
    });
    Object.entries(style).forEach(([key, value]: any[]) => {
      this.compEl.style[key] = value;
    });
  },
  mouseup() {
    if (this.currComp.layout.margin) return;
    const { layout } = this.currComp;

    const offset: any = layout.offset || (layout.offset = {});

    offset[direction[0]] = getOffset.call(this, direction);
  },
});

function getOffset(this: TransferCtrl, direction: Direction) {
  const { transEvent } = this;
  const { layout } = this.currComp;
  let offset = (layout.offset as any)?.[direction[0]] || 0;

  switch (direction) {
    case "top":
      offset += transEvent.offsetY * 2;
      break;
    case "bottom":
      offset += transEvent.offsetY * 2;
      break;
    case "x":
      offset += transEvent.offsetX * 2;
      break;
  }
  return offset;
}

export const offset_top = offset.bind(null, "top");
export const offset_bottom = offset.bind(null, "bottom");
export const offset_left = offset.bind(null, "x");
export const offset_right = offset_left;
